@@ -194,7 +194,7 @@ next_files_sync_check (GDaemonFileEnumerator *enumerator)
   g_mutex_unlock (&enumerator->next_files_mutex);
 }
 
-static void
+static gboolean
 handle_done (GVfsDBusEnumerator *object,
              GDBusMethodInvocation *invocation,
              gpointer user_data)
@@ -209,9 +209,11 @@ handle_done (GVfsDBusEnumerator *object,
   g_signal_emit (enumerator, signals[CHANGED], 0);
 
   gvfs_dbus_enumerator_complete_done (object, invocation);
+
+  return TRUE;
 }
 
-static void
+static gboolean
 handle_got_info (GVfsDBusEnumerator *object,
                  GDBusMethodInvocation *invocation,
                  GVariant *arg_infos,
@@ -248,6 +250,8 @@ handle_got_info (GVfsDBusEnumerator *object,
   g_signal_emit (enumerator, signals[CHANGED], 0);
 
   gvfs_dbus_enumerator_complete_got_info (object, invocation);
+
+  return TRUE;
 }
 
 static void
@@ -393,10 +397,7 @@ _g_task_return_pointer_idle (GTask *task, gpointer result, GDestroyNotify notify
   g_object_set_data (G_OBJECT (task), "_g_task_return_pointer_idle_notify", notify);
 
   source = g_idle_source_new ();
-  g_source_set_priority (source, g_task_get_priority (task));
-  g_source_set_callback (source, (GSourceFunc) _g_task_return_pointer_idle_cb,
-                         g_object_ref (task), g_object_unref);
-  g_source_attach (source, g_task_get_context (task));
+  g_task_attach_source (task, source, (GSourceFunc) _g_task_return_pointer_idle_cb);
   g_source_unref (source);
 }
 
