@@ -95,6 +95,15 @@ check_permission (GVfsBackendAdmin *self,
   invocation = dbus_job->invocation;
   connection = g_dbus_method_invocation_get_connection (invocation);
   credentials = g_dbus_connection_get_peer_credentials (connection);
+  if (!credentials)
+    {
+      g_warning ("The admin backend doesn't work with the session bus "
+                 "fallback. Your application is probably missing "
+                 "--filesystem=xdg-run/gvfsd privileges.");
+      g_vfs_job_failed_literal (job, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                                _("Operation not supported"));
+      return FALSE;
+    }
 
   pid = g_credentials_get_unix_pid (credentials, &error);
   if (error != NULL)
@@ -1094,8 +1103,10 @@ acquire_caps (uid_t uid)
 }
 
 static char *session_address = NULL;
+static char *runtime_dir = NULL;
 static GOptionEntry entries[] = {
   { "address", 0, 0, G_OPTION_ARG_STRING, &session_address, "DBus session address", NULL },
+  { "dir", 0, 0, G_OPTION_ARG_STRING, &runtime_dir, "Runtime dir", NULL },
   { NULL }
 };
 
@@ -1134,4 +1145,7 @@ g_vfs_backend_admin_pre_setup (int *argc,
 
   acquire_caps (uid);
   g_setenv ("DBUS_SESSION_BUS_ADDRESS", session_address, TRUE);
+
+  if (runtime_dir)
+    g_setenv ("XDG_RUNTIME_DIR", runtime_dir, TRUE);
 }
