@@ -264,9 +264,19 @@ auth_callback (SMBCCTX *context,
 
           g_debug ("auth_callback - asking for password...\n");
 
-	  /* translators: First %s is a share name, second is a server name */
-	  message = g_strdup_printf (_("Password required for share %s on %s"),
-				     share_name, server_name);
+          if (backend->user)
+            {
+              /* Translators: First %s is a share name, second is a server name */
+              message = g_strdup_printf (_("Authentication Required\nEnter password for share “%s” on “%s”:"),
+                                         share_name, server_name);
+            }
+          else
+            {
+              /* Translators: First %s is a share name, second is a server name */
+              message = g_strdup_printf (_("Authentication Required\nEnter user and password for share “%s” on “%s”:"),
+                                         share_name, server_name);
+            }
+
 	  handled = g_mount_source_ask_password (backend->mount_source,
 						 message,
 						 username_out,
@@ -1590,6 +1600,7 @@ do_query_fs_info (GVfsBackend *backend,
 
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, "cifs");
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE, TRUE);
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USE_PREVIEW, G_FILESYSTEM_PREVIEW_TYPE_IF_ALWAYS);
 
   if (g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_SIZE) ||
