@@ -367,9 +367,19 @@ auth_callback (SMBCCTX *context,
 
           g_debug ("auth_callback - asking for password...\n");
 
-	  /* translators: %s is a server name */
-	  message = g_strdup_printf (_("Password required for %s"),
-				     server_name);
+          if (backend->user)
+            {
+              /* Translators: %s is a server name */
+              message = g_strdup_printf (_("Authentication Required\nEnter password for “%s”:"),
+                                         server_name);
+            }
+          else
+            {
+              /* Translators: %s is a server name */
+              message = g_strdup_printf (_("Authentication Required\nEnter user and password for “%s”:"),
+                                         server_name);
+            }
+
 	  handled = g_mount_source_ask_password (backend->mount_source,
 						 message,
 						 username_out,
@@ -1496,6 +1506,7 @@ try_query_fs_info (GVfsBackend *backend,
 {
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, "cifs");
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE, TRUE);
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USE_PREVIEW, G_FILESYSTEM_PREVIEW_TYPE_NEVER);
   g_vfs_job_succeeded (G_VFS_JOB (job));
   return TRUE;
 }
